@@ -12,6 +12,25 @@ from pysmac.smacparse import parse_smac_trajectory_string
 
 import numpy as np
 
+def check_java_version():
+    import re
+    from subprocess import STDOUT, check_output
+    out = check_output(["java", "-version"], stderr=STDOUT).split("\n")
+    if len(out) < 1:
+        print "Failed checking Java version. Make sure Java version 7 or greater is installed."
+        return False
+    m = re.match('java version "\d+.(\d+)..*', out[0])
+    if m is None or len(m.groups()) < 1:
+        print "Failed checking Java version. Make sure Java version 7 or greater is installed."
+        return False
+    java_version = int(m.group(1))
+    if java_version < 7:
+        error_msg = "Found Java version %d, but Java version 7 or greater is required." % java_version
+ 
+        raise RuntimeError(error_msg)
+check_java_version()
+
+
 class SMACRunner(object):
     """
         Interface to the SMAC library:
@@ -25,6 +44,7 @@ class SMACRunner(object):
                  x0_int, xmin_int, xmax_int,
                  x_categorical,
                  port, max_evaluations, seed,
+                 cutoff_time,
                  rf_num_trees,
                  rf_full_tree_bootstrap,
                  intensification_percentage
@@ -42,6 +62,7 @@ class SMACRunner(object):
             port: the port to communicate with SMACRemote
             max_evaluations: the maximum number of evaluations
 
+            cutoff_time: CPU time before cut off
             rf_num_trees: number of trees to create in random forest.
             rf_full_tree_bootstrap: bootstrap all data points into trees.
             intensification_percentage: percent of time to spend intensifying versus model learning.
@@ -60,6 +81,7 @@ class SMACRunner(object):
         self._max_evaluations = max_evaluations
         self._seed = seed
 
+        self._cutoff_time = cutoff_time
         self._rf_num_trees = rf_num_trees
         self._rf_full_tree_bootstrap = rf_full_tree_bootstrap
         self._intensification_percentage = intensification_percentage
@@ -84,7 +106,7 @@ class SMACRunner(object):
             Has the SMAC run finished?
         """
         self._smac_process.poll()
-        if self._smac_process.returncode == None:
+        if self._smac_process.returncode is None:
             return False
         else:
             return True
@@ -115,7 +137,8 @@ class SMACRunner(object):
         """
         parameters = {'working_dir': self._working_dir,
                       'exec_dir': self._exec_dir,
-                      'out_dir': self._out_dir}
+                      'out_dir': self._out_dir,
+                      'cutoff_time': self._cutoff_time}
         fdata = """
 algo = echo 0
 execdir = %(exec_dir)s
@@ -124,7 +147,7 @@ deterministic = 1
 rungroup = result
 run_obj = quality
 overall_obj = mean
-cutoff_time = 18000
+cutoff_time = %(cutoff_time)d
 cutoff_length = max
 validation = false
 paramfile = %(working_dir)s/params.pcs
@@ -155,7 +178,7 @@ instance_file = %(working_dir)s/instances.txt
             param_file.write("\n".join(param_definitions))
 
     def _smac_classpath(self):
-        smac_folder = resource_filename(__name__, 'smac/%s/' % SMACRunner.SMAC_VERSION)
+        smac_folder = resource_filename("pysmac", 'smac/%s' % SMACRunner.SMAC_VERSION)
         smac_conf_folder = os.path.join(smac_folder, "conf")
         smac_patches_folder = os.path.join(smac_folder, "patches")
         smac_lib_folder = os.path.join(smac_folder, "lib")
