@@ -150,16 +150,16 @@ public class ExistingAlgorithmRun extends AbstractAlgorithmRun {
 		{ 
 			Object[] args = { execConfig, runConfig, result} ;
 			
-			log.info("Malformed Run Result for Execution (ArrayIndexOutOfBoundsException): {}, Instance: {}, Result: {}", args);
-			log.info("Exception:",e);
+			log.debug("Malformed Run Result for Execution (ArrayIndexOutOfBoundsException): {}, Instance: {}, Result: {}", args);
+			log.debug("Exception:",e);
 			this.setAbortResult(e.getMessage());
 		}catch(NumberFormatException e)
 		{
 			//There was a problem with the output, we just set this flag
 
 			Object[] args = { execConfig, runConfig, result} ;
-			log.info("Malformed Run Result for Execution (NumberFormatException): {}, Instance: {}, Result: {}", args);
-			log.info("Exception:",e);
+			log.debug("Malformed Run Result for Execution (NumberFormatException): {}, Instance: {}, Result: {}", args);
+			log.debug("Exception:",e);
 			this.setAbortResult( e.getMessage());
 			
 			
